@@ -1,4 +1,4 @@
-# Copyright 2013-2017 DataStax, Inc.
+# Copyright DataStax, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,13 +23,13 @@ from datetime import datetime, timedelta
 import re
 import struct
 import time
-import six
-from six.moves import range, zip
+import warnings
 
 from cassandra import ConsistencyLevel, OperationTimedOut
 from cassandra.util import unix_time_from_uuid1
 from cassandra.encoder import Encoder
 import cassandra.encoder
+from cassandra.policies import ColDesc
 from cassandra.protocol import _UNSET_VALUE
 from cassandra.util import OrderedDict, _sanitize_identifiers
 
@@ -75,7 +75,7 @@ def tuple_factory(colnames, rows):
         >>> session = cluster.connect('mykeyspace')
         >>> session.row_factory = tuple_factory
         >>> rows = session.execute("SELECT name, age FROM users LIMIT 1")
-        >>> print rows[0]
+        >>> print(rows[0])
         ('Bob', 42)
 
     .. versionchanged:: 2.0.0
@@ -83,6 +83,39 @@ def tuple_factory(colnames, rows):
     """
     return rows
 
+class PseudoNamedTupleRow(object):
+    """
+    Helper class for pseudo_named_tuple_factory. These objects provide an
+    __iter__ interface, as well as index- and attribute-based access to values,
+    but otherwise do not attempt to implement the full namedtuple or iterable
+    interface.
+    """
+    def __init__(self, ordered_dict):
+        self._dict = ordered_dict
+        self._tuple = tuple(ordered_dict.values())
+
+    def __getattr__(self, name):
+        return self._dict[name]
+
+    def __getitem__(self, idx):
+        return self._tuple[idx]
+
+    def __iter__(self):
+        return iter(self._tuple)
+
+    def __repr__(self):
+        return '{t}({od})'.format(t=self.__class__.__name__,
+                                  od=self._dict)
+
+
+def pseudo_namedtuple_factory(colnames, rows):
+    """
+    Returns each row as a :class:`.PseudoNamedTupleRow`. This is the fallback
+    factory for cases where :meth:`.named_tuple_factory` fails to create rows.
+    """
+    return [PseudoNamedTupleRow(od)
+            for od in ordered_dict_factory(colnames, rows)]
+
 
 def named_tuple_factory(colnames, rows):
     """
@@ -98,16 +131,16 @@ def named_tuple_factory(colnames, rows):
         >>> user = rows[0]
 
         >>> # you can access field by their name:
-        >>> print "name: %s, age: %d" % (user.name, user.age)
+        >>> print("name: %s, age: %d" % (user.name, user.age))
         name: Bob, age: 42
 
         >>> # or you can access fields by their position (like a tuple)
         >>> name, age = user
-        >>> print "name: %s, age: %d" % (name, age)
+        >>> print("name: %s, age: %d" % (name, age))
         name: Bob, age: 42
         >>> name = user[0]
         >>> age = user[1]
-        >>> print "name: %s, age: %d" % (name, age)
+        >>> print("name: %s, age: %d" % (name, age))
         name: Bob, age: 42
 
     .. versionchanged:: 2.0.0
@@ -116,6 +149,20 @@ def named_tuple_factory(colnames, rows):
     clean_column_names = map(_clean_column_name, colnames)
     try:
         Row = namedtuple('Row', clean_column_names)
+    except SyntaxError:
+        warnings.warn(
+            "Failed creating namedtuple for a result because there were too "
+            "many columns. This is due to a Python limitation that affects "
+            "namedtuple in Python 3.0-3.6 (see issue18896). The row will be "
+            "created with {substitute_factory_name}, which lacks some namedtuple "
+            "features and is slower. To avoid slower performance accessing "
+            "values on row objects, Upgrade to Python 3.7, or use a different "
+            "row factory. (column names: {colnames})".format(
+                substitute_factory_name=pseudo_namedtuple_factory.__name__,
+                colnames=colnames
+            )
+        )
+        return pseudo_namedtuple_factory(colnames, rows)
     except Exception:
         clean_column_names = list(map(_clean_column_name, colnames))  # create list because py3 map object will be consumed by first attempt
         log.warning("Failed creating named tuple for results with column names %s (cleaned: %s) "
@@ -139,7 +186,7 @@ def dict_factory(colnames, rows):
         >>> session = cluster.connect('mykeyspace')
         >>> session.row_factory = dict_factory
         >>> rows = session.execute("SELECT name, age FROM users LIMIT 1")
-        >>> print rows[0]
+        >>> print(rows[0])
         {u'age': 42, u'name': u'Bob'}
 
     .. versionchanged:: 2.0.0
@@ -197,8 +244,7 @@ class Statement(object):
     keyspace = None
     """
     The string name of the keyspace this query acts on. This is used when
-    :class:`~.TokenAwarePolicy` is configured for
-    :attr:`.Cluster.load_balancing_policy`
+    :class:`~.TokenAwarePolicy` is configured in the profile load balancing policy.
 
     It is set implicitly on :class:`.BoundStatement`, and :class:`.BatchStatement`,
     but must be set explicitly on :class:`.SimpleStatement`.
@@ -228,7 +274,8 @@ class Statement(object):
                  is_idempotent=False):
         if retry_policy and not hasattr(retry_policy, 'on_read_timeout'):  # just checking one method to detect positional parameter errors
             raise ValueError('retry_policy should implement cassandra.policies.RetryPolicy')
-        self.retry_policy = retry_policy
+        if retry_policy is not None:
+            self.retry_policy = retry_policy
         if consistency_level is not None:
             self.consistency_level = consistency_level
         self._routing_key = routing_key
@@ -279,8 +326,8 @@ class Statement(object):
         return self._serial_consistency_level
 
     def _set_serial_consistency_level(self, serial_consistency_level):
-        acceptable = (None, ConsistencyLevel.SERIAL, ConsistencyLevel.LOCAL_SERIAL)
-        if serial_consistency_level not in acceptable:
+        if (serial_consistency_level is not None and
+                not ConsistencyLevel.is_serial(serial_consistency_level)):
             raise ValueError(
                 "serial_consistency_level must be either ConsistencyLevel.SERIAL "
                 "or ConsistencyLevel.LOCAL_SERIAL")
@@ -384,6 +431,7 @@ class PreparedStatement(object):
     """
 
     column_metadata = None  #TODO: make this bind_metadata in next major
+    retry_policy = None
     consistency_level = None
     custom_payload = None
     fetch_size = FETCH_SIZE_UNSET
@@ -393,12 +441,14 @@ class PreparedStatement(object):
     query_string = None
     result_metadata = None
     result_metadata_id = None
+    column_encryption_policy = None
     routing_key_indexes = None
     _routing_key_index_set = None
-    serial_consistency_level = None
+    serial_consistency_level = None  # TODO never used?
 
     def __init__(self, column_metadata, query_id, routing_key_indexes, query,
-                 keyspace, protocol_version, result_metadata, result_metadata_id):
+                 keyspace, protocol_version, result_metadata, result_metadata_id,
+                 column_encryption_policy=None):
         self.column_metadata = column_metadata
         self.query_id = query_id
         self.routing_key_indexes = routing_key_indexes
@@ -407,14 +457,17 @@ class PreparedStatement(object):
         self.protocol_version = protocol_version
         self.result_metadata = result_metadata
         self.result_metadata_id = result_metadata_id
+        self.column_encryption_policy = column_encryption_policy
         self.is_idempotent = False
 
     @classmethod
     def from_message(cls, query_id, column_metadata, pk_indexes, cluster_metadata,
                      query, prepared_keyspace, protocol_version, result_metadata,
-                     result_metadata_id):
+                     result_metadata_id, column_encryption_policy=None):
         if not column_metadata:
-            return PreparedStatement(column_metadata, query_id, None, query, prepared_keyspace, protocol_version, result_metadata, result_metadata_id)
+            return PreparedStatement(column_metadata, query_id, None,
+                                     query, prepared_keyspace, protocol_version, result_metadata,
+                                     result_metadata_id, column_encryption_policy)
 
         if pk_indexes:
             routing_key_indexes = pk_indexes
@@ -440,7 +493,7 @@ class PreparedStatement(object):
 
         return PreparedStatement(column_metadata, query_id, routing_key_indexes,
                                  query, prepared_keyspace, protocol_version, result_metadata,
-                                 result_metadata_id)
+                                 result_metadata_id, column_encryption_policy)
 
     def bind(self, values):
         """
@@ -488,6 +541,7 @@ class BoundStatement(Statement):
         """
         self.prepared_statement = prepared_statement
 
+        self.retry_policy = prepared_statement.retry_policy
         self.consistency_level = prepared_statement.consistency_level
         self.serial_consistency_level = prepared_statement.serial_consistency_level
         self.fetch_size = prepared_statement.fetch_size
@@ -527,6 +581,7 @@ class BoundStatement(Statement):
             values = ()
         proto_version = self.prepared_statement.protocol_version
         col_meta = self.prepared_statement.column_metadata
+        ce_policy = self.prepared_statement.column_encryption_policy
 
         # special case for binding dicts
         if isinstance(values, dict):
@@ -573,7 +628,13 @@ class BoundStatement(Statement):
                     raise ValueError("Attempt to bind UNSET_VALUE while using unsuitable protocol version (%d < 4)" % proto_version)
             else:
                 try:
-                    self.values.append(col_spec.type.serialize(value, proto_version))
+                    col_desc = ColDesc(col_spec.keyspace_name, col_spec.table_name, col_spec.name)
+                    uses_ce = ce_policy and ce_policy.contains_column(col_desc)
+                    col_type = ce_policy.column_type(col_desc) if uses_ce else col_spec.type
+                    col_bytes = col_type.serialize(value, proto_version)
+                    if uses_ce:
+                        col_bytes = ce_policy.encrypt(col_desc, col_bytes)
+                    self.values.append(col_bytes)
                 except (TypeError, struct.error) as exc:
                     actual_type = type(value)
                     message = ('Received an argument of invalid type for column "%s". '
@@ -754,7 +815,7 @@ class BatchStatement(Statement):
         Like with other statements, parameters must be a sequence, even
         if there is only one item.
         """
-        if isinstance(statement, six.string_types):
+        if isinstance(statement, str):
             if parameters:
                 encoder = Encoder() if self._session is None else self._session.encoder
                 statement = bind_params(statement, parameters, encoder)
@@ -838,10 +899,8 @@ For example::
 
 
 def bind_params(query, params, encoder):
-    if six.PY2 and isinstance(query, six.text_type):
-        query = query.encode('utf-8')
     if isinstance(params, dict):
-        return query % dict((k, encoder.cql_encode_all_types(v)) for k, v in six.iteritems(params))
+        return query % dict((k, encoder.cql_encode_all_types(v)) for k, v in params.items())
     else:
         return query % tuple(encoder.cql_encode_all_types(v) for v in params)
 
@@ -946,7 +1005,8 @@ class QueryTrace(object):
                 SimpleStatement(self._SELECT_SESSIONS_FORMAT, consistency_level=query_cl), (self.trace_id,), time_spent, max_wait)
 
             # PYTHON-730: There is race condition that the duration mutation is written before started_at the for fast queries
-            is_complete = session_results and session_results[0].duration is not None and session_results[0].started_at is not None
+            session_row = session_results.one() if session_results else None
+            is_complete = session_row is not None and session_row.duration is not None and session_row.started_at is not None
             if not session_results or (wait_for_complete and not is_complete):
                 time.sleep(self._BASE_RETRY_SLEEP * (2 ** attempt))
                 attempt += 1
@@ -956,7 +1016,6 @@ class QueryTrace(object):
             else:
                 log.debug("Fetching parital trace info for trace ID: %s", self.trace_id)
 
-            session_row = session_results[0]
             self.request_type = session_row.request
             self.duration = timedelta(microseconds=session_row.duration) if is_complete else None
             self.started_at = session_row.started_at
@@ -1036,3 +1095,17 @@ class TraceEvent(object):
 
     def __str__(self):
         return "%s on %s[%s] at %s" % (self.description, self.source, self.thread_name, self.datetime)
+
+
+# TODO remove next major since we can target using the `host` attribute of session.execute
+class HostTargetingStatement(object):
+    """
+    Wraps any query statement and attaches a target host, making
+    it usable in a targeted LBP without modifying the user's statement.
+    """
+    def __init__(self, inner_statement, target_host):
+            self.__class__ = type(inner_statement.__class__.__name__,
+                                  (self.__class__, inner_statement.__class__),
+                                  {})
+            self.__dict__ = inner_statement.__dict__
+            self.target_host = target_host
